package com.hys.app.controller.erp.goods;

import com.hys.app.framework.database.WebPage;
import com.hys.app.model.erp.dto.*;
import com.hys.app.model.erp.vo.GoodsSnVO;
import com.hys.app.model.erp.vo.GoodsVO;
import com.hys.app.model.support.LogClient;
import com.hys.app.model.support.validator.annotation.Log;
import com.hys.app.service.erp.GoodsManager;
import com.hys.app.service.erp.GoodsSnManager;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 商品序列号相关api
 *
 * @author 张崧
 * 2023-12-26 15:56:58
 */
@RestController
@RequestMapping("/admin/erp/goods/sn")
@Api(tags = "商品序列号相关api")
@Validated
public class GoodsSnController {

    @Autowired
    private GoodsSnManager goodsSnManager;

    @ApiOperation(value = "分页列表")
    @GetMapping
    public WebPage<GoodsSnVO> list(GoodsSnQueryParams queryParams) {
        return goodsSnManager.list(queryParams);
    }


    @ApiOperation(value = "获取统计数据")
    @GetMapping("/count")
    public WebPage<GoodsSnVO> searchCount(GoodsSnQueryParams queryParams) {
        return goodsSnManager.searchCount(queryParams);
    }




    @ApiOperation(value = "添加")
    @PostMapping("/create")
    @Log(client = LogClient.admin, detail = "新增序列号")
    public void add(@RequestBody @Valid GoodsSnDTO goodsDTO) {
        goodsSnManager.add(goodsDTO);
    }

    @ApiOperation(value = "修改")
    @PostMapping("/update/{id}")
    @Log(client = LogClient.admin, detail = "修改id为[${id}]的商品")
    public void edit(@PathVariable Long id, @RequestBody @Valid GoodsSnDTO goodsDTO) {
        goodsDTO.setId(id);
        goodsSnManager.edit(goodsDTO);
    }


    @ApiOperation(value = "批量生成")
    @PostMapping("/save")
    @Log(client = LogClient.admin, detail = "批量生成")
    public void save( @RequestBody @Valid GoodsSnCreateDTO createReq) {
        goodsSnManager.createMore(createReq);
    }


    @ApiOperation(value = "获取详情")
    @GetMapping("/info/{id}")
    public GoodsSnVO getDetail(@PathVariable Long id) {
        return goodsSnManager.getDetail(id);
    }

    @ApiOperation(value = "删除")
    @DeleteMapping("/delete/{id}")
    @Log(client = LogClient.admin, detail = "删除id为${id}的商品")
    public void delete(@PathVariable Long id) {
        goodsSnManager.delete(id);
    }
}

