package com.hys.app.controller.erp.warehouse;

import com.hys.app.framework.database.WebPage;
import com.hys.app.model.erp.dos.WarehouseAreaDO;
import com.hys.app.model.erp.dto.WarehouseAreaQueryParams;
import com.hys.app.model.erp.dto.WarehouseAreaRequest;
import com.hys.app.model.erp.vo.WarehouseAreaVO;
import com.hys.app.model.support.LogClient;
import com.hys.app.model.support.validator.annotation.Log;
import com.hys.app.service.erp.WarehouseAreaManager;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 出库单明细相关API
 *
 * @author 张崧
 * @since 2023-12-07 17:06:32
 */
@RestController
@RequestMapping("/admin/erp/warehouse/area")
@Api(tags = "库位管理相关接口")
@Validated
public class WarehouseAreaController {

    @Autowired
    private WarehouseAreaManager warehouseAreaManager;

    @ApiOperation(value = "获取库位列表")
    @GetMapping("/query")
    public WebPage<WarehouseAreaVO> list(WarehouseAreaQueryParams queryParams) {
        return warehouseAreaManager.list(queryParams);
    }


    @ApiOperation(value = "添加")
    @PostMapping("/create")
    @Log(client = LogClient.admin, detail = "创建库位信息")
    public void add(@RequestBody @Valid WarehouseAreaDO warehouseAreaDO) {
        warehouseAreaManager.add(warehouseAreaDO);
    }

    @ApiOperation(value = "更新")
    @PostMapping("/update")
    @Log(client = LogClient.admin, detail = "创建库位信息")
    public void update(@RequestBody @Valid WarehouseAreaDO warehouseAreaDO) {
        warehouseAreaManager.edit(warehouseAreaDO);
    }

    @ApiOperation(value = "库位详情")
    @PostMapping("/info")
    @Log(client = LogClient.admin, detail = "创建库位信息")
    public WarehouseAreaVO info(@RequestBody @Valid WarehouseAreaRequest request) {
        return warehouseAreaManager.getDetail(request.getId());
    }


    @ApiOperation(value = "删除")
    @DeleteMapping("/delete")
    @Log(client = LogClient.admin, detail = "创建库位信息")
    public void delete(@RequestBody @Valid WarehouseAreaRequest request) {
        warehouseAreaManager.delete(request.getId());
    }
}

