package com.hys.app.service.kuaidiniao;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.URLUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.hys.app.converter.erp.OrderConverter;
import com.hys.app.framework.context.user.AdminUserContext;
import com.hys.app.framework.exception.ServiceException;
import com.hys.app.framework.security.model.Admin;
import com.hys.app.framework.util.DateUtil;
import com.hys.app.model.erp.dos.OrderDO;
import com.hys.app.service.erp.OrderManager;
import com.hys.app.service.kuaidiniao.entity.*;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;

@Service
public class KuaiDiNiaoServiceImpl implements KuaiDiNiaoService {

    /**
     * 快递鸟apiId
     */
    @Value("${kuaidianiao.id}")
    private String id;
    /**
     * 快递鸟apiKey
     */
    @Value("${kuaidianiao.key}")
    private String apiKey;
    /**
     * 快递鸟请求地址
     */
    @Value("${kuaidianiao.apiUrl}")
    private String url;

    @Autowired
    private OrderManager orderManager;

    @Autowired
    private OrderConverter converter;


    @Override
    public String kdShip(PreOrderModel preOrderModel, Long orderId) {
        fillParam(preOrderModel);
        String kdnCode = sendUrl(JSONUtil.toJsonStr(preOrderModel), KdConstants.preOrder);
        if (StrUtil.isNotEmpty(kdnCode)) {
            //如果快递鸟下单成功保存 快递信息
            OrderDO aDo = orderManager.getById(orderId);
            converter.copyProperties(aDo, preOrderModel, preOrderModel.getReceiver(), preOrderModel.getSender());
            aDo.setKdnOrderCode(kdnCode);
            orderManager.updateById(aDo);
        }
        return kdnCode;
    }


    /**
     * @param dataJson      业务参数
     * @param interfaceType 请求接口指令
     * @return 返回JSON
     */
    private String sendUrl(String dataJson, String interfaceType) {
        try {
            String design = URLEncoder.encode(Base64.encode(DigestUtil.md5Hex(dataJson + apiKey)), "UTF-8");
            //拼接参数
            String param = "interfaceType=" + interfaceType + "&customerCode=" + id + "&requestData=" + dataJson + "&dataSign=" + design;
            //发请求
            String res = HttpUtil.post(url, param);
            Map map = JSON.parseObject(res, Map.class);
            //resultCode == 100表示预下单成功，预下单郑工后立马调用下单接口
            if (ObjectUtil.isNotNull(map.get("resultCode")) && 100 == Integer.valueOf(map.get("resultCode").toString())) {
                //拼接参数
                String createParam = "interfaceType=" + KdConstants.createOrder + "&customerCode=" + id + "&requestData=" + dataJson + "&dataSign=" + design;
                //发请求
                String resCreate = HttpUtil.post(url, createParam);
                Map mapCreate = JSON.parseObject(resCreate, Map.class);
                //resultCode == 100表示下单成功，预下单成功后立马调用下单接口
                if (ObjectUtil.isNotNull(mapCreate.get("resultCode")) && 100 == Integer.valueOf(mapCreate.get("resultCode").toString())) {
                    Map mapData = JSON.parseObject(mapCreate.get("data").toString(), Map.class);
                    return mapData.get("kdnOrderCode").toString();
                }
            } else {
                throw new ServiceException(map.get("reason").toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException("请求快递鸟发货异常，请联系客服");
        }
        return null;
    }

    private void fillParam(PreOrderModel preOrderModel) {
        //填充送货人的经纬度
        LongitudeLatitude longitudeLatitudeSender = GaoDeUtil.getLongitudeLatitude(preOrderModel.getSender().getProvinceCityAreaAddress());
        preOrderModel.getSender().setLongitude(longitudeLatitudeSender.getLongitude());
        preOrderModel.getSender().setLatitude(longitudeLatitudeSender.getLatitude());
        //填充收货人的经纬度
        LongitudeLatitude longitudeLatitudeReceiver = GaoDeUtil.getLongitudeLatitude(preOrderModel.getSender().getProvinceCityAreaAddress());
        preOrderModel.getReceiver().setLongitude(longitudeLatitudeReceiver.getLongitude());
        preOrderModel.getReceiver().setLatitude(longitudeLatitudeReceiver.getLatitude());
        //获取下单人信息
        preOrderModel.setOrderName(AdminUserContext.getAdminUserName());
        preOrderModel.setOrderPhone("13127911751");


    }

    /**
     * 推送待接单状态	接口指令103，状态码100
     * 推送业务员接单状态	接口指令103，状态码201
     * 推送业务员到达状态	接口指令103，状态码202
     * 推送配送中状态	接口指令103，状态码300
     * 推送签收状态
     * (签收分为两种签收)	接口指令103，状态码401（正常签收）
     * 接口指令103，状态码402（退回签收）
     * 推送取消状态	接口指令103，状态码500
     * 推送下单失败状态	接口指令103，状态码600
     * 推送订单费用状态	接口指令103，状态码206
     */

    @Override
    public resReturnBackBase kdnReturnBack(ReturnBackBase returnBackBase) {
        Map map = JSON.parseObject(returnBackBase.getRequestData(), Map.class);
        resReturnBackBase base = new resReturnBackBase();
        base.setKdnOrderCode(map.get("kdnOrderCode").toString());
        //验签
        OrderDO aDo = orderManager.getById(Integer.valueOf(map.get("orderCode").toString()));
        if (!checkDesign(aDo, returnBackBase.getDataSign())) {
            base.setRequestData("验签失败");
            base.setResultCode(1002);
            return base;
        }


        switch (map.get("interfaceType").toString()) {
            case "100":

                break;
            case "201":
                break;
            case "202":
                break;
            case "300":
                break;
            case "401":
                break;
            case "402":
                break;
            case "500":
                break;
            case "600":
                ReCreateOrderFail reCreateOrderFail = JSON.parseObject(returnBackBase.getRequestData(), ReCreateOrderFail.class);
                break;
            case "206":
                break;
            default:
                base.setResultCode(1002);
                base.setReason("内部业务异常,麻烦尽快通知商家,tel:13127911751");
        }
        return base;
    }

    private Boolean checkDesign(OrderDO aDo, String dataSign) {
        try {
            PreOrderModel model = new PreOrderModel();
            converter.copyProperties(model, aDo);
            String design = URLEncoder.encode(Base64.encode(DigestUtil.md5Hex(JSONUtil.toJsonStr(model) + apiKey)), "UTF-8");
            if (dataSign.equals(design)) {
                return true;
            }
            return false;
        } catch (Exception e) {
            return false;
        }
    }
}
